function drawChart(datos,title,id) {
  var custom_width=$('#'+id).parent().width()*0.95;
  var custom_height=$(window).height()/3*0.60;

  // Set chart options
  var options = {title:title,
                 width: custom_width,
                 height:custom_height,
                 fontSize: 10,
                 hAxis: {
                    viewWindow:{
                        min:0
                    }
                 },
                chartArea:{left:'10%',right:'10%'},
                };


  // Instantiate and draw our chart, passing in some options.
  var view = new google.visualization.DataView(datos);
  view.setColumns([0, 1,
                       { calc: "stringify",
                         sourceColumn: 1,
                         type: "string",
                         role: "annotation" }]);
  var chart = new google.visualization.BarChart(document.getElementById(id));
  google.visualization.events.addListener(chart, 'ready', ready_charts);
  chart.draw(view, options);
}

function get_sizes(id){
    var custom_width=$('#'+id).parent().width()*0.90;
    var custom_height=$(window).height()*0.72/2;
    return {'width':custom_width,'height':custom_height};
}

function build_data_per_month_releases(data,title){
    var total=[];
    for (i=0 ; i < 12; i++){
	total[i]={'15':0,'16':0,'all':0};
	if (! data[i]){
	    continue;
	}
        for (rel in data[i]){
            if (rel == '15'){
                for (fla in data[i][rel]){
                    total[i][rel]+=data[i][rel][fla];
                }
            }
            if (rel == '16'){
                for (fla in data[i][rel]){
                    total[i][rel]+=data[i][rel][fla];
                }
            }
        }
        total[i]['all']=total[i]['15']+total[i]['16']; 
    }
    var dates_axis=[];
    var d= new Date();
    var data_table = new google.visualization.DataTable();
    data_table.addColumn({type:'date',role:'domain'},'Release');
    data_table.addColumn('number','Total Release 15');
    data_table.addColumn({type:'string',role:'style'},'Style Release 15');
    data_table.addColumn('number','Total Release 16');
    data_table.addColumn({type:'string',role:'style'},'Style Release 16');
    data_table.addColumn('number','Total ALL Releases');
    data_table.addColumn({type:'string',role:'style'},'Style ALL Releases');
    date_values=[];
    for (i=1;i <= 12;i++){
        month=d.getMonth();
        year=d.getFullYear();
        date_to_add=new Date(year,month)
        dates_axis.push(date_to_add);
        date_values.push([date_to_add,total[i-1]['15'],'color:blue',total[i-1]['16'],'color:red',total[i-1]['all'],'color:grey'])
        d.setMonth(month-1);
    }
    data_table.addRows(date_values);
    var options= {  title: title, 
                    curveType: 'function', 
                    isStacked: false,
                    hAxis:{title:'Months',ticks:dates_axis,format:'MMM'},
                    vAxis:{title:'Num hosts'},
                    legend:{textStyle:{fontSize: 12}},
                    chartArea:{left:'10%',right:'10%'},
                    focusTarget: 'category',
                    colors:['blue','red','grey'],
                    lineWidth: 5,
                    };
    return {data:data_table,options:options}
}
function build_data_per_month_flavours(data,title){
    var total=[];
    for (i=0 ; i < 12; i++){
        total[i]={'server':0,'client':0,'desktop':0,'other':0,'all':0};
        if (! data[i]){
            continue;
        }
        for (rel in data[i]){
            for (fla in data[i][rel]){
                total[i][fla]+=data[i][rel][fla];
                total[i]['all']+=data[i][rel][fla];
            }
        }
    }
    var dates_axis=[];
    var d= new Date();
    var data_table = new google.visualization.DataTable();
    data_table.addColumn({type:'date',role:'domain'},'Flavour');
    data_table.addColumn('number','Total Server');
    data_table.addColumn({type:'string',role:'style'},'Style Server');
    data_table.addColumn('number','Total Client');
    data_table.addColumn({type:'string',role:'style'},'Style Client');
    data_table.addColumn('number','Total Desktop');
    data_table.addColumn({type:'string',role:'style'},'Style Desktop');
    data_table.addColumn('number','Total Other');
    data_table.addColumn({type:'string',role:'style'},'Style Other');
    data_table.addColumn('number','Total ALL Flavours');
    data_table.addColumn({type:'string',role:'style'},'Style Total ALL Flavours');
    date_values=[];
    for (i=1;i <= 12;i++){
        month=d.getMonth();
        year=d.getFullYear();
        date_to_add=new Date(year,month)
        dates_axis.push(date_to_add);
        date_values.push([date_to_add,total[i-1]['server'],'color:purple',total[i-1]['client'],'color:teal',total[i-1]['desktop'],'color:gold',total[i-1]['other'],'color:olive',total[i-1]['all'],'color:grey'])
        d.setMonth(month-1);
    }
    data_table.addRows(date_values);
    var options= {  title: title, 
                    curveType: 'function', 
                    isStacked: false,
                    hAxis:{title:'Months',ticks:dates_axis,format:'MMM'},
                    vAxis:{title:'Num hosts'},
                    legend:{textStyle:{fontSize: 12}},
                    chartArea:{left:'10%',right:'10%'},
                    focusTarget: 'category',
                    colors: ['purple','teal','gold','olive','grey'],
                    lineWidth: 5,
                    };
    return {data:data_table,options:options}
}
function build_data_from_array(data,title){
    var dates_axis=[];
    var d= new Date();
    var data_table = new google.visualization.DataTable();
    data_table.addColumn({type:'date',role:'domain'},'Nhosts');
    data_table.addColumn('number',title);
    date_values=[];
    for (i=1;i <= 12;i++){
        month=d.getMonth();
        year=d.getFullYear();
        date_to_add=new Date(year,month)
        dates_axis.push(date_to_add);
        date_values.push([date_to_add,data[i-1]])
        d.setMonth(month-1);
    }
    data_table.addRows(date_values);
    var options= {  title: title, 
                    curveType: 'function', 
                    isStacked: false,
                    hAxis:{title:'Months',ticks:dates_axis,format:'MMM'},
                    vAxis:{title:'Num hosts',minValue: 0,maxValue:'automatic',viewWindowMode:'pretty',viewWindow:{min:0,max:'auto'}},
                    legend:{textStyle:{fontSize: 12}},
                    chartArea:{left:'10%',right:'10%'},
                    focusTarget: 'category',
                    lineWidth: 5,
                    };
    return {data:data_table,options:options}
}
function doExtendedChart(){
    end_load_time=get_time();
    var parameter=window.location.search.substr(1);
    if (parameter != ''){
        parameter='?'+parameter;
    }
    var chart_data={};
    start_get_time=get_time();
    $('#accordion').append('<div id="loading"></div>');
    $.getJSON( "./GetExtendedStats"+parameter, function( json ) {
    $('#loading').remove();
    end_get_time=get_time();
    start_graph_time=end_get_time;
    var app=null;
    for (key in json){
        if (key == 'clients'){
            for (type in json[key]){
                switch(type){
                    case 'clients_per_month':
                        chart_data[type]={};
                        chart_data[type]['title']='Machines';
                        chart_data[type]['releases']={};
                        chart_data[type]['releases']['graph']=build_data_per_month_releases(json[key][type],'Number machines by release per month')
                        chart_data[type]['releases']['title']='Number machines by release per month';
                        chart_data[type]['flavours']={};
                        chart_data[type]['flavours']['graph']=build_data_per_month_flavours(json[key][type],'Number machines by flavour per month')
                        chart_data[type]['flavours']['title']='Number machines by flavour per month';
                        break;
                    case 'freq_updates_per_month':
                        chart_data[type]={};
                        chart_data[type]['title']='Updates';
                        chart_data[type]['releases']={};
                        chart_data[type]['releases']['graph']=build_data_per_month_releases(json[key][type],'Number updates by release per month')
                        chart_data[type]['releases']['title']='Number updates by release per month';
                        chart_data[type]['flavours']={};
                        chart_data[type]['flavours']['graph']=build_data_per_month_flavours(json[key][type],'Number updates by flavour per month')
                        chart_data[type]['flavours']['title']='Number updates by flavour per month';
                        break;
                    case 'change_releases':
                        if (! chart_data['changes']){
                            chart_data['changes']={};
                            chart_data['changes']['title']='Changes';
                        }
                        if (! chart_data['changes']['releases'])
                            chart_data['changes']['releases']={};
                        chart_data['changes']['releases']['graph']=build_data_from_array(json[key][type],'Change releases per month')
                        chart_data['changes']['releases']['title']='Change releases per month';
                        break;
                    case 'change_flavours':
                        if (! chart_data['changes']){
                            chart_data['changes']={};
                            chart_data['changes']['title']='Changes';
                        }
                        if (! chart_data['changes']['flavours'])
                            chart_data['changes']['flavours']={};
                        chart_data['changes']['flavours']['graph']=build_data_from_array(json[key][type],'Change flavours per month')
                        chart_data['changes']['flavours']['title']='Change flavours per month';
                        break;
                }
            }
        }
        if (key == 'apps'){
            for (type in json['apps']){
                switch(type){
                    case 'app':
                        app=json['apps'][type];
                        break;
                    case 'app_use':
                        if (json[key][type]){
                            chart_data[type]={};
                            chart_data[type]['title']='Application usage';
                            chart_data[type]['releases']={};
                            chart_data[type]['releases']['graph']=build_data_per_month_releases(json[key][type],'Application use by release per month ('+app+')');
                            chart_data[type]['releases']['title']='Use per month for application '+app;
                            chart_data[type]['flavours']={};
                            chart_data[type]['flavours']['graph']=build_data_per_month_flavours(json[key][type],'Application use by flavour per month ('+app+')');
                            chart_data[type]['flavours']['title']='Use per month for application '+app;
                        }
                        break;
                }
            }
        }
        if (key.startsWith('debug')){
            if (key == 'debug_query_time'){
                query_time=json[key];
            }
            if (key == 'debug_keep_alive'){
                keep_alive=json[key];
            }
        }
    }

    for (chart_type in chart_data){
        if (! chart_data[chart_type]['title']){
            continue;
        }

        for (chart_variant in chart_data[chart_type]){
            if (chart_variant == 'title'){
                $('#accordion').append('<h3 class="'+chart_type+'">'+chart_data[chart_type]['title']+'</h3><div id="'+chart_type+'"></div>');
                continue;
            }
            if (chart_data[chart_type][chart_variant]['graph']){
                $('#'+chart_type).append('<div id='+chart_type+'_'+chart_variant+'></div>');
                var sizes=get_sizes(chart_type);
                chart_data[chart_type][chart_variant]['graph']['options']['width']=sizes['width'];
                chart_data[chart_type][chart_variant]['graph']['options']['height']=sizes['height'];
                var chart = new google.visualization.LineChart(document.getElementById(chart_type+'_'+chart_variant));
                chart.draw(chart_data[chart_type][chart_variant]['graph']['data'], chart_data[chart_type][chart_variant]['graph']['options']);
            }
        }
    }


    $('#accordion').accordion({active:false,collapsible:true});
    if (app != null){
        $('.app_use').click();
    }
    end_graph_time=get_time();
    print_stats(end_graph_time,query_time);

    });
    return;
}
function doChart(){
    end_load_time=get_time();
    var datos1= new google.visualization.DataTable();
    datos1.addColumn('string','App');
    datos1.addColumn('number','Count');

    var datos2= new google.visualization.DataTable();
    datos2.addColumn('string','App');
    datos2.addColumn('number','Count');

    var datos3= new google.visualization.DataTable();
    datos3.addColumn('string','App');
    datos3.addColumn('number','Count');

    start_get_time=get_time();

    //Gets Data from DB
    $('#accordion').append('<div id="loading"></div>');
    $.getJSON( "./GetStats", function( json ) {
        $('#loading').remove();
        end_get_time=get_time();
        start_graph_time=end_get_time;
        count=0;
        for (key in json){
            if (! key.startsWith('debug')){
                for (key2 in json[key]){
                    count++;
                }
            }
        }
        max_graph=count*3;
        obj=[];
        for (distro in json){
            if (distro.startsWith('debug')){
                if (distro == 'debug_query_time'){
                    query_time=json[distro];
                }
                if (distro == 'debug_keep_alive'){
                    keep_alive=json[distro];
                }
                continue;
            }
            for (sabor in json[distro]){
                titles=['Top apps este mes','Top apps ultimo mes','Top apps penultimo mes'];
                divname='chart_'+distro+'_'+sabor;
                $('#accordion').append('<h3>Distro '+distro+'('+sabor+')</h3>');
                $('#accordion').append('<div class="'+divname+'"></h3>');
                for (i=0; i<3; i++){
                    $('div.'+divname).append('<div id="'+divname+i+'"></div>');
                    k=0;
                    var datos=new google.visualization.DataTable();
                    datos.addColumn('string','App');
                    datos.addColumn('number','Count');
                    $.each(json[distro][sabor][i][0],function(key,value){
                        //console.log( i+':'+k+'->'+key + " : " + value );
                        datos.addRow([key,parseInt(value)]);
                        k++;
                    })
                    while (k < 10){
                        datos.addRow(['',0]);
                        k++;
                    }
                    obj.push(datos);
                    //console.log('drawing:'+titles[i]+' into '+divname+i);
                    titles[i]=titles[i]+' (Total clients: '+json[distro][sabor][i][1]['nclients']+')'
                    drawChart(obj[obj.length-1],titles[i],divname+i);
                }
            }
        }
        end_graph_time=get_time();
        print_stats(end_graph_time,query_time);
        return;
    });
}
num_ready=0;
function ready_charts(){
    num_ready++;
    if (num_ready == max_graph){
        $('#accordion').accordion({active:false,collapsible:true});
    }
}
function print_stats(gr_time=0,q_time=0){
    $('#stats_box').append('<span class="stats">Load time: '+((end_load_time-start_load_time)/1000).toString()+' sec</span>')
    $('#stats_box').append('<span class="stats">Get time: '+parseFloat(((end_get_time-start_get_time)/1000)-q_time).toFixed(3).toString()+' sec</span>')
    $('#stats_box').append('<span class="stats">Graph time: '+((gr_time-start_graph_time)/1000).toString()+' sec</span>')
    $('#stats_box').append('<span class="stats">Query time: '+parseFloat(q_time).toFixed(3).toString()+' sec</span>')
    $('#stats_box').append('<span class="stats">Updated: '+keep_alive.toString()+'</span>')
}
function get_time(){
    return (Date.now ? Date.now() : new Date().getTime()) ;
}
var start_graph_time=0;
var end_graph_time=0;
var start_load_time=get_time();
var end_load_time=0;
var start_get_time=0;
var end_get_time=0;
var query_time='';
var keep_alive='';